<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 742 $
 * $Author: Nouvelle $
 * $Date: 2023-11-12 00:55:54 +0100 (So, 12 Nov 2023) $
 */

defined("mxMainFileLoaded") or die("access denied");

/**
 * Globales Kategorien System
 *
 * * Alle Ausgaben (return) beziehen sich nur auf das eigene Modul (addon)
 * * Fehler werden im Debug-Modus angezeigt
 *
 * @package pragmaMx
 * @class pmxCategory
 * @author Sven Hedström-Lang <svenhlang@gmail.com>
 * @copyright 27/10/2023
 * @since 2.9.0
 * @phpversion 8.2.4
 * @require class pmxBase, MX_IS_SYSADMIN
 * @files
 * * includes/classes/Category.php (this file)
 * * setup/systabledefs/_category.php
 *
 * TODO: Evtl. Datenbank per cron bereinigen und Kategorien ohne Oberkategorien entfernen
 *
 * @function
 * * getAll()
 * * add()
 * * update()
 * * delete()
 * * getCount()
 * * exists()
 * * refresh()
 */
class pmxCategory
{
  /**
   * Der Tabellename
   * @var string
   * @default {$prefix}_category
   */
  private string $_dbName;

  /**
   * Die Tabelle als Array zur weiteren Verarbeitung
   * @var array
   * @default []
   */
  private array $_dbArray = [];

  /**
   * Um welches Addon/Modul es sich handelt
   * @require
   * @var string
   */
  private string $_addon;

  /**
   * Damit können mehrere Kategorien je Modul unterschieden werden
   * @var string
   * @default ""
   */
  private string $_type = "";

  /**
   * @param string $addon - Erforderlich
   * @param string $type - Optional
   * @return void
   * @example
   *    load_class("Category", PMX_MODULE);
   *    // oder
   *    $cat = load_class("Category", PMX_MODULE, "Favoriten");
   *    $cat->getAll(); // Gibt alle Kategorien aus
   */
  public function __construct(string $addon, ?string $type = "")
  {
    $this->_dbName = pmxBase::prefix() . "_category";
    $this->_addon = $addon;
    $this->_type = $type;
    $this->_initArray();
  }

  /**
   * Aktualisiert die Daten, die ausgegeben werden aus der Datenbank
   * zb. wenn was geändert wurde
   *
   * @alias _initArray()
   * @return void
   */
  public function refresh()
  {
    $this->_initArray();
  }

  /**
   * Gibt alle Kategorien als Array aus
   *
   * @return array - Alle Kategorien aus dem Modul - Alphabetisch sortiert
   *
   * * string "addon" - Das benutzte Modul
   * * array "subtitle" - Array mit all den Ober-Kategorien die enthalten sind
   * * array "subtitleid" - Array mit den entsprechenden ids von "subtitle"
   * * bool "isparent" - true = Wenn es keine weitere Unterkategorien gibt, ansonsten false
   *
   * @example
   *    pmxCategory::getAll();
   *    // Ausgabe
   *    array[
   *      array[
   *        "id"          => "1",
   *        "parentid"    => "0",
   *        "title"       => "PMX",
   *        "description" => "Die Haupt Kategorie",
   *        "image"       => "pmx.png",
   *        "subtitle"    => ["PMX"],
   *        "subtitleid"  => ["1"],
   *        "addon"       => "Marketplace",
   *        "type"        => "Favoriten",
   *        "isparent"    => false
   *      ],
   *      array[
   *        "id"          => "3",
   *        "parentid"    => "1",
   *        "title"       => "Module",
   *        "description" => "Alle Module",
   *        "image"       => "module.png",
   *        "subtitle"    => ["PMX", "Module"],
   *        "subtitleid"  => ["1", "3"],
   *        "addon"       => "Marketplace",
   *        "type"        => "Favoriten",
   *        "isparent"    => true
   *      ]
   *    ]
   */
  public function getAll(): array
  {
    return $this->_dbArray;
  }

  /**
   * Neue Kategorien hinzufügen
   *
   * @param array $arr
   * @param string $arr["title"]        Erforderlich
   * @param int $arr["parentid"]        Optional
   * @param string $arr["description"]  Optional
   * @param string $arr["image"]        Optional
   * @return void
   * @example
   *    pmxCategory::add(["title" => "Archiv"]);
   *    pmxCategory::add(["title" => "Altes Archiv", "parentid" => 12]);
   */
  public function add(array $arr): void
  {
    $arr["parentid"] = isset($arr["parentid"]) ? $arr["parentid"] : 0;
    $arr["title"] = isset($arr["title"]) ? $arr["title"] : "";
    $arr["description"] = isset($arr["description"]) ? $arr["description"] : "";
    $arr["image"] = isset($arr["image"]) ? $arr["image"] : "";

    if (strlen($arr["title"]) > 0) {
      sql_query(
        "INSERT IGNORE INTO $this->_dbName VALUES (NULL, '" .
          $arr["parentid"] .
          "', '" .
          $arr["title"] .
          "', '" .
          $arr["description"] .
          "', '" .
          $arr["image"] .
          "', '" .
          $this->_addon .
          "', '" .
          $this->_type .
          "');"
      );
    }
  }

  /**
   * Aktualisieren der Kategorie
   *
   * Wenn keine optionalen Parameter angegeben sind, werden diese auch nicht geändert
   *
   * TODO: Hier wir irgendwann eine Fehlermeldung kommen, weil nur bei "parentid" kein Komma am Anfang gesetzt ist
   *
   * @param array $arr
   * @param int $arr["id"] - Erforderlich - Die ID der Kategorie, die geändert werden soll
   * @param int $arr["parentid"] - Optional
   * @param string $arr["title"] - Optional
   * @param string $arr["description"] - Optional
   * @param string $arr["image"] - Optional
   * @return void
   * @example
   *    pmxCategory::update(["id" => 7, "parentid" => 5, "title" => "Links"]);
   */
  public function update(array $arr): void
  {
    $arr["id"] = isset($arr["id"]) ? $arr["id"] : null;

    if ($arr["id"]) {
      $query = "";
      $query .= isset($arr["parentid"])
        ? "parentid = '" . $arr["parentid"] . "'"
        : "";
      $query .= isset($arr["title"]) ? ", title = '" . $arr["title"] . "'" : "";
      $query .= isset($arr["description"])
        ? ', description = "' . $arr["description"] . '"'
        : "";
      $query .= isset($arr["image"]) ? ", image = '" . $arr["image"] . "'" : "";
      $where =
        " id = '" .
        $arr["id"] .
        "' AND addon = '" .
        $this->_addon .
        "' AND type = '" .
        $this->_type .
        "'";
      if (strlen($query) > 0) {
        sql_query(
          "UPDATE IGNORE {$this->_dbName} SET {$query} WHERE {$where} LIMIT 1"
        );
      }
    }
  }

  /**
   * Entfernt Kategorien.
   *
   * TODO: Wenn gelöscht wird, sollte die Kategorie keine Unterkategorien besitzen - wird hier nicht abgefragt.
   *
   * @param array $arr["id"] - Optional - Die ID's der Kategorien, die gelöscht werden sollen.
   * @return void
   * @example
   *    // Löscht alle die zum Modul gehören
   *    pmxCategory::delete();
   *
   *    // Löscht alle nur mit den "id" und zum Modul gehören
   *    pmxCategory::delete(["id" => [47, 36, 59]]);
   */
  public function delete(array $arr): void
  {
    if (MX_IS_SYSADMIN) {
      $arr["id"] = isset($arr["id"]) ? array_unique($arr["id"]) : [];
      $ids = implode("','", $arr["id"]);

      $where = " addon = '$this->_addon' AND type = '$this->_type'";
      $where .= count($arr["id"]) ? " AND id IN ($ids)" : "";

      // TODO: Evtl als Einstellung im Adminbereich ? Das hier fremde Module keinen mist bauen
      // delete DB
      sql_query("DELETE FROM {$this->_dbName} WHERE {$where}");
    }
  }

  /**
   * Gibt die Anzahl der Kategorien inklusive Unter-Kategorien als Zahl aus
   *
   * @return int
   */
  public function getCount(): int
  {
    return count($this->_dbArray);
  }

  /**
   * Schauen ob überhaupt Kategorien existieren
   *
   * @return bool - ja "true", nein "false"
   */
  public function exists(): bool
  {
    return count($this->_dbArray) > 0;
  }

  /**
   * Speichert die Datenbank in einem Array, für die lokale Verarbeitung
   *
   * @private
   * @return void
   */
  private function _initArray(): void
  {
    $ids = [];
    $parentids = [];
    $this->_dbArray = [];

    // Daten aus der DB holen
    $sql = "SELECT * FROM $this->_dbName WHERE addon='$this->_addon' AND type='$this->_type'";
    $result = sql_query($sql);
    while ($value_1 = sql_fetch_array($result)) {
      $ids[] = $value_1["id"];
      $parentids[] = $value_1["parentid"];
      $this->_dbArray[] = [
        "id" => $value_1["id"],
        "parentid" => $value_1["parentid"],
        "title" => $value_1["title"],
        "description" => $value_1["description"],
        "image" => $value_1["image"],
        "subtitle" => [$value_1["title"]], // array
        "subtitleid" => [$value_1["id"]], // array
        "addon" => $value_1["addon"],
        "type" => $value_1["type"],
        // "isparent" => null
      ];
    }

    // Suchen der subtitle und subtitleid
    foreach ($this->_dbArray as $key_2 => $value_2) {
      if ($value_2["parentid"] !== "0") {
        $this->_initSub([
          "keyid" => $key_2,
          "parentid" => $value_2["parentid"],
        ]);
      }
    }

    // Aussortieren von isparent
    $different = array_diff($ids, $parentids);
    // Alle suchen die keine Unterkategorien haben (true)
    foreach ($this->_dbArray as $key => $value) {
      $this->_dbArray[$key]["isparent"] = in_array($value["id"], $different);
    }
    // Alphabetisch sortieren
    $sort = array_column($this->_dbArray, "subtitle");
    array_multisort($sort, SORT_ASC, $this->_dbArray);
    // TODO: nochmal sortieren
    // https://wiki.php.net/rfc/combined-comparison-operator
    // https://stackoverflow.com/questions/3232965/sort-multidimensional-array-by-multiple-columns
  }

  /**
   * Fügt der _initArray() alle "subtitle" und "subtitleid" hinzu
   *
   * @private
   * @param array $_arr
   * @param string $_arr["keyid"] - Der INDEX im Array
   * @param string $_arr["parentid"]
   * @return void
   */
  private function _initSub(array $_arr): void
  {
    if ($_arr["parentid"] !== "0") {
      foreach ($this->_dbArray as $_value) {
        if ($_arr["parentid"] === $_value["id"]) {
          array_unshift(
            $this->_dbArray[$_arr["keyid"]]["subtitle"],
            $_value["title"]
          );
          array_unshift(
            $this->_dbArray[$_arr["keyid"]]["subtitleid"],
            $_value["id"]
          );
          if ($_value["parentid"] !== 0) {
            // weitersuchen
            $this->_initSub([
              "keyid" => $_arr["keyid"],
              "parentid" => $_value["parentid"],
            ]);
          }
          break;
        }
      }
    }
  }
}
