<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 748 $
 * $Author: Nouvelle $
 * $Date: 2023-11-15 19:48:22 +0100 (Mi, 15 Nov 2023) $
 */

if (!defined("mxMainFileLoaded")) {
  die("You can't access this file directly..");
}

/**
 * Cookie per PHP
 *
 * @autor Sven Hedstroem-Lang <svenhlang@gmail.com>
 * @copyright 2022-04
 * @since pmx 2.8.1
 * @phpversion >=8.0.1
 * @api https://www.php.net/manual/de/reserved.variables.cookies.php
 * @api https://www.php.net/manual/de/function.setcookie.php
 */
class pmxCookie
{
  /**
   * Die Klasse sollte vor dem html geladen werden
   *
   * @since pmx 2.8.1
   */
  public function __construct()
  {
    // nothing
  }

  /**
   * Set the Cookie
   *
   * INFO: $value kann nur (string|boolean|int) evtl andere Typen hinzufügen
   *
   * @since pmx 2.8.1
   * @api https://www.php.net/manual/de/function.setcookie.php
   * @param string $name - Ungültige Zeichen werden mit einem Unterstrich "_" ersetzt
   * @param string|bool|int $value
   * @param int [$seconds=1] - Optional - Default 1, set in seconds - Specifies when the cookie expires. If this parameter is omitted or set to 0, the cookie will expire at the end of the session (when the browser closes).
   * @param string [$path=""] - Optional
   * @param string [$domain=""] - Optional
   * @param bool [$secure=false] - Optional
   * @param bool [$httponly=false] - Optional
   * @return void
   * @example
   *      // remove cookie when closed the browser
   *      pmxCookie::setCookie("tab_seo", 1);
   *
   *      // set cookie for 5 days
   *      pmxCookie::setCookie("tab_seo", 3, 60*60*24*5);
   */
  public static function set(
    string $name,
    string|bool|int $value,
    ?int $seconds = 1,
    ?string $path = "",
    ?string $domain = "",
    ?bool $secure = false,
    ?bool $httponly = false
  ) {
    $newname = str_replace(
      [" ", ".", ",", ";", "\\", "\t", "\r", "\n", "\013", "\014"],
      "_",
      $name
    );

    setcookie(
      htmlspecialchars($newname),
      $value,
      time() + $seconds,
      $path,
      $domain,
      $secure,
      $httponly
    );
  }

  /**
   * Gibt den Wert des Cookie aus
   *
   * pmxCookie::get()
   *
   * @since pmx 2.8.1
   * @return string|bool|int - returns empty string "" if not found
   */
  public static function get(string $name): string|bool|int
  {
    $tmp = htmlspecialchars($name);
    return isset($_COOKIE[$tmp]) ? $_COOKIE[$tmp] : "";
  }

  /**
   * Löscht den Cookie
   *
   * @since pmx 2.8.1
   * @return bool
   */
  public static function remove(string $name): bool
  {
    $tmp = htmlspecialchars($name);
    if (isset($_COOKIE[$tmp])) {
      unset($_COOKIE[$tmp]);
      self::set($tmp, "", 1);
      return true;
    }
    return false;
  }

  /**
   * @since pmx 2.8.1
   * @return bool
   */
  public static function exists($name)
  {
    return isset($_COOKIE[htmlspecialchars($name)]);
  }

  /**
   * Gibt alle Cookies als array aus
   *
   * @since pmx 2.9.0
   * @return array
   */
  public static function getAll(): array
  {
    return $_COOKIE;
  }
}
